import { __decorate } from 'tslib';
import { Input, ViewChild, Component, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';

var CanvasInputComponent_1;
const VECTOR_X = 0;
const VECTOR_Y = 1;
const VECTOR_DRAG = 2;
const VECTOR_SIZE = 3;
const VECTOR_COLOR = 4;
const VECTOR_OPACITY = 5;
const defaultCanvasModel = {
    width: 500,
    height: 800,
    vectors: [],
    imgUrl: '',
};
let CanvasInputComponent = CanvasInputComponent_1 = class CanvasInputComponent {
    constructor() {
        this.color = '#ff0000';
        this.size = 5; // natural numbers
        this.opacity = 1; // 0-1 intervals
        this.offsetTop = 0;
        this.offsetLeft = 0;
        this.paint = false;
        this.width = 400;
        this.height = 400;
        this.vectors = []; // [x, y, drag, size, color, opacity]
        this.imgLoaded = false;
        this.disabled = false;
        this.viewInitiated = false;
        this.onChangeListeners = [];
        this.onBlurListeners = [];
    }
    ngAfterViewInit() {
        this.paper = this.paperRef.nativeElement;
        this.ctx = this.paper.getContext('2d');
        if (!this.ctx) {
            throw new Error('browser not supporting canvas');
        }
        this.paper.ontouchstart = (e) => this.onTouchStart(e);
        this.paper.onmousedown = (e) => this.onMouseDown(e);
        this.paper.onmousemove = (e) => this.onMouseMove(e);
        this.paper.ontouchmove = (e) => this.onTouchMove(e);
        this.paper.ontouchend = (e) => this.disablePaint(e);
        this.paper.onmouseup = (e) => this.disablePaint(e);
        this.paper.onmouseleave = (e) => this.disablePaint(e);
        this.viewInitiated = true;
        // draw image & vectors
        this.registerOnChange((val) => this.writeValue(val));
    }
    // clean ups
    ngOnDestroy() {
        this.onChangeListeners = [];
        this.onBlurListeners = [];
        this.paper.ontouchstart = null;
        this.paper.onmousedown = null;
        this.paper.onmousemove = null;
        this.paper.ontouchmove = null;
        this.paper.ontouchend = null;
        this.paper.onmouseup = null;
        this.paper.onmouseleave = null;
    }
    /**
     * Writes a new value to the element.
     *
     * This method is called by the forms API to write to the view when programmatic
     * changes from model to view are requested.
     */
    writeValue(value) {
        var _a;
        if (!value) {
            return null;
        }
        if (this.width !== value.width) {
            this.width = value.width;
            this.paper.width = this.width;
        }
        if (this.height !== value.height) {
            this.height = value.height;
            this.paper.height = this.height;
        }
        this.vectors = value.vectors;
        this.imgUrl = value.imgUrl;
        if (((_a = this.img) === null || _a === void 0 ? void 0 : _a.src) !== value.imgUrl) {
            this.loadImage();
        }
        this._reDraw();
    }
    loadImage() {
        var _a;
        if (!this.viewInitiated) {
            return null;
        }
        if (!((_a = this.imgUrl) === null || _a === void 0 ? void 0 : _a.length)) {
            return null;
        }
        const w = this.width;
        const h = this.height;
        this.imgLoaded = false;
        this.img = new Image();
        this.img.onload = () => {
            this.imgLoaded = true;
            this.ctx.drawImage(this.img, 0, 0, w, h);
        };
        this.img.src = this.imgUrl;
    }
    /**
     * Registers a callback function that is called when the control's value
     * changes in the UI.
     *
     * This method is called by the forms API on initialization to update the form
     * model when values propagate from the view to the model.
     */
    registerOnChange(fn) {
        if (!fn) {
            return null;
        }
        this.onChangeListeners.push(fn);
    }
    /**
     * Registers a callback function is called by the forms API on initialization
     * to update the form model on blur.
     */
    registerOnTouched(fn) {
        if (!fn) {
            return null;
        }
        this.onBlurListeners.push(fn);
    }
    /**
     * Function that is called by the forms API when the control status changes to
     * or from 'DISABLED'. Depending on the status, it enables or disables the
     * appropriate DOM element.
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    onMouseDown(e) {
        if (this.disabled) {
            return null;
        }
        e.preventDefault();
        const x = e.pageX - this.paper.offsetLeft + this.offsetLeft;
        const y = e.pageY - this.paper.offsetTop + this.offsetTop;
        this.paint = true;
        this.addPoint(x, y);
    }
    onTouchStart(e) {
        if (this.disabled) {
            return null;
        }
        e.preventDefault();
        const x = e.touches[0].pageX - this.paper.offsetLeft + this.offsetLeft;
        const y = e.touches[0].pageY - this.paper.offsetTop + this.offsetTop;
        this.paint = true;
        this.addPoint(x, y);
    }
    onMouseMove(e) {
        if (this.disabled) {
            return null;
        }
        e.preventDefault();
        if (!this.paint) {
            return null;
        }
        const x = e.pageX - this.paper.offsetLeft + this.offsetLeft;
        const y = e.pageY - this.paper.offsetTop + this.offsetTop;
        this.addPoint(x, y, true);
    }
    onTouchMove(e) {
        if (this.disabled) {
            return null;
        }
        e.preventDefault();
        if (!this.paint) {
            return null;
        }
        const x = e.touches[0].pageX - this.paper.offsetLeft + this.offsetLeft;
        const y = e.touches[0].pageY - this.paper.offsetTop + this.offsetTop;
        this.addPoint(x, y, true);
    }
    disablePaint(e) {
        var _a;
        e.preventDefault();
        this.paint = false;
        // update on blur handler
        const $event = {
            width: this.width,
            height: this.height,
            vectors: [...this.vectors],
            imgUrl: (_a = this.img) === null || _a === void 0 ? void 0 : _a.src,
        };
        this.onBlurListeners.forEach((fn) => fn($event));
    }
    addPoint(x, y, drag = false) {
        var _a;
        const size = this.size;
        const color = this.color;
        const opacity = this.opacity;
        this.vectors.push([x, y, drag, size, hexToRgb(color), opacity]);
        const $event = {
            width: this.width,
            height: this.height,
            vectors: [...this.vectors],
            imgUrl: (_a = this.img) === null || _a === void 0 ? void 0 : _a.src,
        };
        this.onChangeListeners.forEach((fn) => fn($event));
    }
    _reDraw() {
        if (!this.viewInitiated) {
            return null;
        }
        const w = this.width;
        const h = this.height;
        this.ctx.clearRect(0, 0, w, h);
        if (this.imgLoaded) {
            this.ctx.drawImage(this.img, 0, 0, w, h);
        }
        this.ctx.lineJoin = 'round';
        for (let i = 0; i < this.vectors.length; i++) {
            this.ctx.beginPath();
            if (this.vectors[i][VECTOR_DRAG] && i > 0) {
                this.ctx.moveTo(this.vectors[i - 1][VECTOR_X], this.vectors[i - 1][VECTOR_Y]);
            }
            else {
                this.ctx.moveTo(this.vectors[i][VECTOR_X] - 1, this.vectors[i][VECTOR_Y]);
            }
            this.ctx.lineTo(this.vectors[i][VECTOR_X], this.vectors[i][VECTOR_Y]);
            this.ctx.closePath();
            const { r, g, b } = this.vectors[i][VECTOR_COLOR];
            this.ctx.strokeStyle = `rgba(${r}, ${g}, ${b}, ${this.vectors[i][VECTOR_OPACITY]})`;
            this.ctx.lineWidth = this.vectors[i][VECTOR_SIZE];
            this.ctx.stroke();
        }
    }
};
__decorate([
    Input()
], CanvasInputComponent.prototype, "color", void 0);
__decorate([
    Input()
], CanvasInputComponent.prototype, "size", void 0);
__decorate([
    Input()
], CanvasInputComponent.prototype, "opacity", void 0);
__decorate([
    Input()
], CanvasInputComponent.prototype, "offsetTop", void 0);
__decorate([
    Input()
], CanvasInputComponent.prototype, "offsetLeft", void 0);
__decorate([
    ViewChild('paper')
], CanvasInputComponent.prototype, "paperRef", void 0);
CanvasInputComponent = CanvasInputComponent_1 = __decorate([
    Component({
        selector: 'sirus-canvas-input',
        template: "<canvas #paper></canvas>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => CanvasInputComponent_1),
                multi: true,
            },
        ]
    })
], CanvasInputComponent);
// hexToRgb will convert hex to RGB color format
function hexToRgb(hex) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result
        ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16),
        }
        : null;
}

let CanvasInputModule = class CanvasInputModule {
};
CanvasInputModule = __decorate([
    NgModule({
        declarations: [CanvasInputComponent],
        imports: [FormsModule],
        exports: [CanvasInputComponent],
    })
], CanvasInputModule);

/*
 * Public API Surface of canvas-input-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CanvasInputComponent, CanvasInputModule, defaultCanvasModel };
//# sourceMappingURL=sirusdev-canvas-input-angular.js.map
